;( function( $, undefined ) {
	"use strict";
	var USOFAutocomplete = function( container ) {
		// Variables
		this._separator = ',';
		// Prefix for get params
		this._prefix = 'params:';
		// Delay for search requests
		this._typingDelay = 0.5;
		// Event KeyCodes
		this.keyCodes = {
			ENTER: 13,
			BACKSPACE: 8
		};

		var defaultSettings = {
			_nonce: '',
			slug: '',
			multiple: false,
		};

		// Elements
		this.$container = $( container );
		this.$toggle = $( '.usof-autocomplete-toggle', this.$container );
		this.$options = $( '.usof-autocomplete-options', this.$container );
		this.$search = $( 'input[type="text"]', this.$options );
		this.$removeAll = $( '.usof-autocomplete-remove-all', this.$options );
		this.$list = $( '.usof-autocomplete-list', this.$container );
		this.$message = $( '.usof-autocomplete-message', this.$container );
		this.$value = $( '> .usof-autocomplete-value', this.$container );

		// Load settings
		this.settings = $.extend( defaultSettings, this.$container[ 0 ].onclick() || {} );
		this.$container.removeAttr( 'onclick' );

		// List of all parameters
		this.items = {};
		$( '> *', this.$list ).each( function( _, item ) {
			var $item = $( item );
			this.items[ $item.data( 'value' ) ] = $item;
		}.bind( this ) );

		// Events
		this.$options
			.on( 'click', '.usof-autocomplete-selected-remove', this._events.remove.bind( this ) )
			.on( 'click', '.usof-autocomplete-remove-all', this._events.removeAll.bind( this ) );
		this.$list
			.on( 'click', '> *', this._events.selected.bind( this ) )
			.on( 'scroll', this._events.scroll.bind( this ) );
		this.$search
			.on( 'keyup', this._events.keyup.bind( this ) )
			.on( 'input', this._events.searchDelay.bind( this ) )
			.on( 'focus blur', this._events.toggleList.bind( this ) );

		// Parameters which are not in the list and need to be loaded
		var loadParams = [],
			initValues = ( this.$value.val() || '' ).split( this._separator ) || [];

		if ( initValues.length ) {
			this.$removeAll.addClass( 'show' );
		}

		// Selection of parameters during initialization
		initValues.map( function( key ) {
			if ( !key ) {
				return;
			}
			var $item = $( '[data-value="' + key + '"]:first', this.$list )
				.addClass( 'selected' );
			if ( $item.length ) {
				this.$search
					.before( this.getSelectedTemplate.call( this, key ) );
			} else {
				loadParams.push( key );
			}
		}.bind( this ) );

		// Loading and selection of parameters which are not in the list but must be displayed
		if ( loadParams.length ) {
			this.$search.val( this._prefix + loadParams.join( this._separator ) );
			this.ajax.call( this, function( items ) {
				$.each( items, function( key ) {
					$( '[data-value="' + key + '"]:first', this.$list )
						.addClass( 'selected' );
					this.$search
						.before( this.getSelectedTemplate.call( this, key ) );
				}.bind( this ) )
			}.bind( this ) );
			this.$search.val( '' );
		}
	};
	// Export API
	USOFAutocomplete.prototype = {
		/**
		 * State loaded
		 */
		loaded: false,
		// Handlers
		_events: {
			/**
			 * Remove selected
			 * @param object e jQueryEvent
			 * @return void
			 */
			remove: function( e ) {
				e.preventDefault();
				var $target = $( e.currentTarget ),
					$selected = $target.closest( '.usof-autocomplete-selected' ),
					key = $selected.data( 'key' );
				this._removeValue.call( this, key );
				$( '[data-value="' + key + '"]', this.$list ).removeClass( 'selected' );
				$selected.remove();
				this.$removeAll
					.toggleClass( 'show', !!this.$value.val() );
			},
			/**
			 * Delete all selected options
			 * @param object e jQueryEvent
			 * @return void
			 */
			removeAll: function( e ) {
				e.preventDefault();
				$( '.usof-autocomplete-selected-remove', this.$options )
					.removeClass( 'show' )
					.trigger( 'click' );
			},
			/**
			 * Delayed search to avoid premature queries
			 * @param object e jQueryEvent
			 * @return void
			 */
			searchDelay: function( e ) {
				if ( ! e.currentTarget.value ) {
					return;
				}
				if ( !! this._typingTimer ) {
					clearTimeout( this._typingTimer );
				}
				this._typingTimer = setTimeout( function() {
					this._events.search.call( this, e );
					clearTimeout( this._typingTimer );
				}.bind( this ), 1000 * this._typingDelay );
			},
			/**
			 * Filtering results when entering characters in the search field
			 * @param object e jQueryEvent
			 * @return void
			 */
			search: function( e ) {
				var $input = $( e.currentTarget ),
					value = ( $.trim( $input.val() ).toLowerCase() ).replace( /\=|\"|\s/, '' ),
					$items = $( '> *', this.$list ),
					/**
					 * Filters parameters by search text
					 * @param jQueryObject $items
					 * @return void
					 */
					filter = function( $items ) {
						$items
							.addClass( 'hidden' )
							.filter( '[data-text^="' + value + '"]' )
							.removeClass( 'hidden' );
					};

				// Check value
				if ( !value || value.length < 1 ) {
					$items.removeClass( 'hidden' );
					return;
				}

				// Filter by search text
				filter.call( this, $items );

				// Search preload
				this.ajax.call( this, function( items ) {
					// Filter by search text
					filter.call( this, this.$list.find( '> *' ) );
					// Messages no results found
					if ( value && !$( '> *:not(.hidden)', this.$list ).length ) {
						this.showMessage.call( this, this.settings.no_results_found );
					} else {
						this.clearMessage.call( this );
						this.$toggle.addClass( 'show' );
					}
				}.bind( this ) );
			},
			/**
			 * Selected option
			 * @param object e jQueryEvent
			 * @return void
			 */
			selected: function( e ) {
				var $target = $( e.currentTarget ),
					selectedValue = $target.data( 'value' ) || '';
				if ( $target.hasClass( 'selected' ) ) {
					// Remove item
					this._removeValue.call( this, selectedValue );
					$( '[data-key="' + selectedValue + '"]', this.$options ).remove();
					$target.removeClass( 'selected' );
				} else if ( this._addValue.call( this, selectedValue ) ) {
					if ( ! this.settings.multiple ) {
						$( '.usof-autocomplete-selected', this.$options ).remove();
						$( '> *', this.$list ).removeClass( 'selected' );
						this.$value.val( '' );
					}
					this.$toggle.removeClass( 'show' );
					$target.addClass( 'selected' );
					// Added item
					this.$search
						.val( '' )
						.before( this.getSelectedTemplate.call( this, selectedValue ) );
				}
				this.$removeAll
					.toggleClass( 'show', !!this.$value.val() );
			},
			/**
			 * When scrolling a sheet, load the parameters
			 * @param object e jQueryEvent
			 * @return void
			 */
			scroll: function( e ) {
				var $target = $( e.currentTarget );
				if ( !this.loaded && ( $target.scrollTop() + $target.height() ) >= e.currentTarget.scrollHeight ) {
					this.ajax.call( this );
				}
			},
			/**
			 * Input event handler for Search
			 * @param object e jQueryEvent
			 * @return void
			 */
			keyup: function( e ) {
				if ( e.keyCode === this.keyCodes.ENTER ) {
					// If you press enter and there are matching elements, then selected option.
					var search = $.trim( this.$search.val() ),
						$selected = $( '> [data-text="'+ search +'"]:visible:first', this.$list );
					if ( !$selected.length ) {
						$selected = $( '> *:visible:first', this.$list );
					}
					if ( $selected.length ) {
						$selected.trigger( 'click' );
					}
				}
				if( e.keyCode === this.keyCodes.BACKSPACE ) {
					if ( !$.trim( this.$search.val() ) ) {
						this.clearMessage.call( this );
						this.$toggle.addClass( 'show' );
					}
				}
			},
			/**
			 * Show/Hide list
			 * @param {object} e jQueryEvent
			 * @return void
			 */
			toggleList: function( e ) {
				var isFocus = ( e.type === 'focus' ),
					pid = setTimeout( function() {
						this.$toggle.toggleClass( 'show', isFocus );
						clearTimeout( pid );
					}.bind( this ), ( isFocus ? 0 : 300 ) );
				// If there is no search text, then all parameters are show
				if ( !$.trim( this.$search.val() ) ) {
					$( '> .hidden', this.$list )
						.removeClass( 'hidden' );
				}
			}
		},
		/**
		 * Load and search option
		 * @param function callback
		 * @return void
		 */
		ajax: function( callback ) {
			if ( this.loaded ) {
				return;
			}

			this.loaded = true;
			this.$options.addClass( 'loaded' );
			this.clearMessage.call( this );

			// Request data
			var data = {
				_nonce: this.settings._nonce,
				action: 'us_autocomplete',
				offset: $( '> *:visible', this.$list ).length,
				search: $.trim( this.$search.val() ),
				slug: this.settings.slug
			};

			// Get data
			$.get( ajaxurl, data, function( res ) {
				this.loaded = false;
				this.$options.removeClass( 'loaded' );
				this.clearMessage.call( this );

				if ( !res.success ) {
					this.showMessage.call( this, res.data.message );
					return;
				}

				// Add to the list of new parameters
				$.each( res.data.items, function( value, name ) {
					if ( !this.items.hasOwnProperty( value ) ) {
						var text = ( name || '' ).replace( /\s/, '' ).toLowerCase(),
						$item = $( '<div data-value="'+ value +'" data-text="'+ text +'" tabindex="3">'+ name +'</div>' );
						this.$list.append( $item );
						this.items[ value ] = $item;
					}
				}.bind( this ) );

				// Run callback function
				if ( $.isFunction( callback ) ) {
					callback.call( this, res.data.items );
				}
			}.bind( this ), 'json' );
		},
		/**
		 * Show the message.
		 * @param string text The message text
		 * @return void
		 */
		showMessage: function( text ) {
			this.$list.addClass( 'hidden' );
			this.$message
				.text( text )
				.removeClass( 'hidden' );
		},
		/**
		 * Clear this message
		 * @return void
		 */
		clearMessage: function() {
			this.$list.removeClass( 'hidden' );
			this.$message
				.addClass( 'hidden' )
				.text('');
		},
		/**
		 * Adding a parameter to the result
		 * @param string key The key
		 * @return boolean
		 */
		_addValue: function( key ) {
			var values = ( this.$value.val() || '' ).split( this._separator ),
				isNotEnabled = false;
			for ( var k in values ) {
				if ( values[ k ] === key ) {
					isNotEnabled = true;
					break;
				}
			}
			if ( !isNotEnabled ) {
				values.push( key );
				var value = ( values || [] ).join( this._separator ).replace(/^\,/, '');
				this.$value.val( value );
				return true;
			}
			return false;
		},
		/**
		 * Removing a parameter from the result
		 * @param string key The key
		 * @return void
		 */
		_removeValue: function( key ) {
			var values = ( this.$value.val() || '' ).split( this._separator ),
				index = values.indexOf( key );
			if ( index !== - 1 ) {
				delete values[ index ];
				// Reset indexes
				values = values.filter( function( item ) {
					return item !== undefined;
				} );
				this.$value.val( values.join( this._separator ) );
			}
		},
		/**
		 * Get the selected template.
		 * @param string key The key
		 * @return string
		 */
		getSelectedTemplate: function( key ) {
			var $selected = $( '[data-value="' + key + '"]:first', this.$list );
			if ( !$selected.length ) {
				return '';
			}
			return '<span class="usof-autocomplete-selected" data-key="' + key + '">\
				' + $selected.text() + ' <a href="javascript:void(0)" title="Remove" class="usof-autocomplete-selected-remove"></a>\
			</span>';
		}
	};
	$( '.usof-autocomplete' ).each( function() {
		new USOFAutocomplete( this );
	} );
} )( jQuery );
